@@ -1,10 +1,35 @@
 # dealings with ez_setup ------------------------------------------------------
-import ez_setup
-
-ez_setup.use_setuptools()
+import distribute_setup
+distribute_setup.use_setuptools()
 
+import setuptools
 from setuptools import Extension
 
+if 'distribute' not in setuptools.__file__:
+    print "-------------------------------------------------------------------------"
+    print "Setuptools conflict detected."
+    print "-------------------------------------------------------------------------"
+    print "When I imported setuptools, I did not get the distribute version of"
+    print "setuptools, which is troubling--this package really wants to be used"
+    print "with distribute rather than the old setuptools package. More than likely,"
+    print "you have both distribute and setuptools installed, which is bad."
+    print
+    print "See this page for more information:"
+    print "http://wiki.tiker.net/DistributeVsSetuptools"
+    print "-------------------------------------------------------------------------"
+    print "I will continue after a short while, fingers crossed."
+    print "-------------------------------------------------------------------------"
+
+    delay = 10
+
+    from time import sleep
+    import sys
+    while delay:
+        sys.stdout.write("Continuing in %d seconds...   \r" % delay)
+        sys.stdout.flush()
+        delay -= 1
+        sleep(1)
+
 def setup(*args, **kwargs):
     from setuptools import setup
     import traceback
@@ -15,9 +40,9 @@ def setup(*args, **kwargs):
     except SystemExit:
         raise
     except:
-        print "--------------------------------------------------------------------------"
-        print "Sorry, your build failed. Try rerunning configure with different options."
-        print "--------------------------------------------------------------------------"
+        print "----------------------------------------------------------------------------"
+        print "Sorry, your build failed. Try rerunning configure.py with different options."
+        print "----------------------------------------------------------------------------"
         raise
 
 
@@ -47,17 +72,36 @@ class NumpyExtension(Extension):
 
 
 class PyUblasExtension(NumpyExtension):
-    def get_pyublas_incpath(self):
+    def get_module_include_path(self, name):
         from imp import find_module
-        file, pathname, descr = find_module("pyublas")
-        from os.path import join
-        return join(pathname, "..", "include")
+        file, pathname, descr = find_module(name)
+        from os.path import join, exists
+        installed_path = join(pathname, "..", "include")
+        development_path = join(pathname, "..", "src", "cpp")
+        if exists(installed_path):
+            return installed_path
+        elif exists(development_path):
+            return development_path
+        else:
+            raise RuntimeError("could not find C include path for module '%s'" % name)
 
     @property
     def include_dirs(self):
         return self._include_dirs + [
                 self.get_numpy_incpath(),
-                self.get_pyublas_incpath(),
+                self.get_module_include_path("pyublas"),
+                ]
+
+
+
+
+class HedgeExtension(PyUblasExtension):
+    @property
+    def include_dirs(self):
+        return self._include_dirs + [
+                self.get_numpy_incpath(),
+                self.get_module_include_path("pyublas"),
+                self.get_module_include_path("hedge"),
                 ]
 
 
@@ -85,26 +129,27 @@ def humanize(sym_str):
 
 
 # siteconf handling -----------------------------------------------------------
-def get_config(schema=None):
+def get_config(schema=None, warn_about_no_config=True):
     if schema is None:
         from setup import get_config_schema
         schema = get_config_schema()
 
-    if not schema.have_config() and not schema.have_global_config():
-        print "********************************************************"
-        print "*** I have detected that you have not run configure."
-        print "********************************************************"
+    if (not schema.have_config() and not schema.have_global_config()
+            and warn_about_no_config):
+        print "*************************************************************"
+        print "*** I have detected that you have not run configure.py."
+        print "*************************************************************"
         print "*** Additionally, no global config files were found."
         print "*** I will go ahead with the default configuration."
         print "*** In all likelihood, this will not work out."
         print "*** "
         print "*** See README_SETUP.txt for more information."
         print "*** "
-        print "*** If the build does fail, just re-run configure with the"
+        print "*** If the build does fail, just re-run configure.py with the"
         print "*** correct arguments, and then retry. Good luck!"
-        print "********************************************************"
+        print "*************************************************************"
         print "*** HIT Ctrl-C NOW IF THIS IS NOT WHAT YOU WANT"
-        print "********************************************************"
+        print "*************************************************************"
 
         delay = 10
 
@@ -121,9 +166,18 @@ def get_config(schema=None):
 
 
 
-def hack_distutils(debug=False):
+def hack_distutils(debug=False, fast_link=True):
     # hack distutils.sysconfig to eliminate debug flags
     # stolen from mpi4py
+
+    def remove_prefixes(optlist, bad_prefixes):
+        for bad_prefix in bad_prefixes:
+            for i, flag in enumerate(optlist):
+                if flag.startswith(bad_prefix):
+                    optlist.pop(i)
+                    break
+        return optlist
+
     import sys
     if not sys.platform.lower().startswith("win"):
         from distutils import sysconfig
@@ -131,14 +185,8 @@ def hack_distutils(debug=False):
         cvars = sysconfig.get_config_vars()
         cflags = cvars.get('OPT')
         if cflags:
-            cflags = cflags.split()
-            for bad_prefix in ('-g', '-O', '-Wstrict-prototypes', '-DNDEBUG'):
-                for i, flag in enumerate(cflags):
-                    if flag.startswith(bad_prefix):
-                        cflags.pop(i)
-                        break
-                if flag in cflags:
-                    cflags.remove(flag)
+            cflags = remove_prefixes(cflags.split(),
+                    ['-g', '-O', '-Wstrict-prototypes', '-DNDEBUG'])
             if debug:
                 cflags.append("-g")
             else:
@@ -147,6 +195,13 @@ def hack_distutils(debug=False):
             cvars['OPT'] = str.join(' ', cflags)
             cvars["CFLAGS"] = cvars["BASECFLAGS"] + " " + cvars["OPT"]
 
+        if fast_link:
+            for varname in ["LDSHARED", "BLDSHARED"]:
+                ldsharedflags = cvars.get(varname)
+                if ldsharedflags:
+                    ldsharedflags = remove_prefixes(ldsharedflags.split(),
+                            ['-Wl,-O'])
+                    cvars[varname] = str.join(' ', ldsharedflags)
 
 
 
@@ -159,6 +214,39 @@ def default_or(a, b):
 
 
 
+def expand_str(s, options):
+    import re
+
+    def my_repl(match):
+        sym = match.group(1)
+        try:
+            repl = options[sym]
+        except KeyError:
+            from os import environ
+            repl = environ[sym]
+
+        return expand_str(repl, options)
+
+    return re.subn(r"\$\{([a-zA-Z0-9_]+)\}", my_repl, s)[0]
+
+def expand_value(v, options):
+    if isinstance(v, (str, unicode)):
+        return expand_str(v, options)
+    elif isinstance(v, list):
+        return [expand_value(i, options) for i in v]
+    else:
+        return v
+
+
+def expand_options(options):
+    for k in options.keys():
+        options[k] = expand_value(options[k], options)
+    return options
+
+
+
+
+
 
 class ConfigSchema:
     def __init__(self, options, conf_file="siteconf.py", conf_dir="."):
@@ -172,7 +260,7 @@ class ConfigSchema:
 
         import sys
         if not sys.platform.lower().startswith("win"):
-            self.global_conf_file = expanduser("/etc/aksetup-defaults.py")
+            self.global_conf_file = "/etc/aksetup-defaults.py"
         else:
             self.global_conf_file = None
 
@@ -248,8 +336,12 @@ class ConfigSchema:
 
     def have_global_config(self):
         import os
-        return (os.access(self.user_conf_file, os.R_OK) or 
-                os.access(self.global_conf_file, os.R_OK))
+        result = os.access(self.user_conf_file, os.R_OK)
+
+        if self.global_conf_file is not None:
+            result = result or os.access(self.global_conf_file, os.R_OK)
+
+        return result
 
     def have_config(self):
         import os
@@ -273,6 +365,8 @@ class ConfigSchema:
                     raise KeyError, "invalid config key in %s: %s" % (
                             cfile, key)
 
+        expand_options(result)
+
         return result
 
     def add_to_configparser(self, parser, def_config=None):
@@ -287,6 +381,7 @@ class ConfigSchema:
         result = {}
         for opt in self.options:
             result[opt.name] = opt.take_from_configparser(options)
+        expand_options(result)
         return result
 
     def write_config(self, config):
@@ -404,6 +499,89 @@ class Libraries(StringListOption):
                 help=help or ("Library names for %s (without lib or .so)" 
                 % (human_name or humanize(lib_name))))
 
+class BoostLibraries(Libraries):
+    def __init__(self, lib_base_name):
+        Libraries.__init__(self, "BOOST_%s" % lib_base_name.upper(), 
+                ["boost_%s-${BOOST_COMPILER}-mt" % lib_base_name],
+                help="Library names for Boost C++ %s library (without lib or .so)" 
+                    % humanize(lib_base_name))
+
+def set_up_shipped_boost_if_requested(conf):
+    """Set up the package to use a shipped version of Boost.
+
+    Return a tuple of a list of extra C files to build and extra
+    defines to be used.
+    """
+    from os.path import exists
+    import sys
+
+    if conf["USE_SHIPPED_BOOST"]:
+        if not exists("bpl-subset/bpl_subset/boost/version.hpp"):
+            print >>sys.stderr, "------------------------------------------------------------------------"
+            print >>sys.stderr, "The shipped Boost library was not found, but USE_SHIPPED_BOOST is True."
+            print >>sys.stderr, "(The files should be under bpl-subset/.)"
+            print >>sys.stderr, "------------------------------------------------------------------------"
+            print >>sys.stderr, "If you got this package from git, you probably want to do"
+            print >>sys.stderr, ""
+            print >>sys.stderr, " $ git submodule init"
+            print >>sys.stderr, " $ git submodule update"
+            print >>sys.stderr, ""
+            print >>sys.stderr, "to fetch what you are presently missing. If you got this from"
+            print >>sys.stderr, "a distributed package on the net, that package is broken and"
+            print >>sys.stderr, "should be fixed. For now, I will turn off 'USE_SHIPPED_BOOST'"
+            print >>sys.stderr, "to try and see if the build succeeds that way, but in the long"
+            print >>sys.stderr, "run you might want to either get the missing bits or turn"
+            print >>sys.stderr, "'USE_SHIPPED_BOOST' off."
+            print >>sys.stderr, "------------------------------------------------------------------------"
+            conf["USE_SHIPPED_BOOST"] = False
+
+            delay = 10
+
+            from time import sleep
+            import sys
+            while delay:
+                sys.stdout.write("Continuing in %d seconds...   \r" % delay)
+                sys.stdout.flush()
+                delay -= 1
+                sleep(1)
+
+    if conf["USE_SHIPPED_BOOST"]:
+        conf["BOOST_INC_DIR"] = ["bpl-subset/bpl_subset"]
+        conf["BOOST_LIB_DIR"] = []
+        conf["BOOST_PYTHON_LIBNAME"] = []
+        conf["BOOST_THREAD_LIBNAME"] = []
+
+        from glob import glob
+        source_files = (glob("bpl-subset/bpl_subset/libs/*/*/*/*.cpp")
+                + glob("bpl-subset/bpl_subset/libs/*/*/*.cpp")
+                + glob("bpl-subset/bpl_subset/libs/*/*.cpp"))
+
+        source_files = [f for f in source_files
+                if not f.startswith("bpl-subset/bpl_subset/libs/thread/src")]
+
+        import sys
+        if sys.platform == "nt":
+            source_files += glob(
+                    "bpl-subset/bpl_subset/libs/thread/src/win32/*.cpp")
+        else:
+            source_files += glob(
+                    "bpl-subset/bpl_subset/libs/thread/src/pthread/*.cpp")
+
+        return (source_files, 
+                {"BOOST_MULTI_INDEX_DISABLE_SERIALIZATION": 1}
+                )
+    else:
+        return [], {}
+
+
+def make_boost_base_options():
+    return [
+        IncludeDir("BOOST", []),
+        LibraryDir("BOOST", []),
+        Option("BOOST_COMPILER", default="gcc43", 
+            help="The compiler with which Boost C++ was compiled, e.g. gcc43"),
+        ]
+
 
 
 
@@ -437,7 +615,8 @@ def configure_frontend():
     parser.add_option("--disable-static", help="Ignored", action="store_false")
     parser.add_option("--update-user", help="Update user config file (%s)" % schema.user_conf_file, 
             action="store_true")
-    parser.add_option("--update-global", help="Update global config file (%s)" % schema.global_conf_file, 
+    parser.add_option("--update-global", 
+            help="Update global config file (%s)" % schema.global_conf_file, 
             action="store_true")
 
     schema.add_to_configparser(parser, schema.read_config())
@@ -493,7 +672,7 @@ def substitute(substitutions, fname):
                 l = l[:match.start()] + subst  + l[match.end():]
                 made_change = True
         new_lines.append(l)
-    new_lines.insert(1, "# DO NOT EDIT THIS FILE -- it is generated by configure\n")
+    new_lines.insert(1, "# DO NOT EDIT THIS FILE -- it was generated by configure.py\n")
     import sys
     new_lines.insert(2, "# %s\n" % (" ".join(sys.argv)))
     open(fname, "w").write("".join(new_lines))
