@@ -34,6 +34,7 @@
 #include <textidentificationframe.h>
 #include <uniquefileidentifierframe.h>
 #include <unknownframe.h>
+#include <unsynchronizedlyricsframe.h>
 #include <apetag.h>
 
 #include "common.hpp"
@@ -329,6 +330,20 @@ void exposeID3()
       ;
   }
 
+  {
+    typedef ID3v2::UnsynchronizedLyricsFrame cl;
+    class_<cl, bases<ID3v2::Frame>, boost::noncopyable>
+      ("id3v2_UnsynchronizedLyricsFrame", init<optional<const ByteVector &> >())
+      .def(init<String::Type>())
+      .DEF_SIMPLE_METHOD(language)
+      .DEF_SIMPLE_METHOD(setLanguage)
+      .DEF_SIMPLE_METHOD(description)
+      .DEF_SIMPLE_METHOD(setDescription)
+      .DEF_SIMPLE_METHOD(textEncoding)
+      .DEF_SIMPLE_METHOD(setTextEncoding)
+      ;
+  }
+
   {
     typedef ID3v2::UserTextIdentificationFrame cl;
     class_<cl, bases<ID3v2::TextIdentificationFrame>, boost::noncopyable>
